package com.supervision.police.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.supervision.common.utils.IPages;
import com.supervision.common.utils.ListUtils;
import com.supervision.common.utils.StringUtils;
import com.supervision.minio.domain.MinioFile;
import com.supervision.minio.mapper.MinioFileMapper;
import com.supervision.minio.service.MinioService;
import com.supervision.police.domain.ModelCase;
import com.supervision.police.domain.ModelRecordType;
import com.supervision.police.domain.NoteRecordSplit;
import com.supervision.police.domain.NoteRecord;
import com.supervision.police.mapper.ModelCaseMapper;
import com.supervision.police.mapper.ModelRecordTypeMapper;
import com.supervision.police.mapper.NoteRecordSplitMapper;
import com.supervision.police.mapper.NoteRecordMapper;
import com.supervision.police.service.ModelRecordTypeService;
import com.supervision.police.service.NoteRecordSplitService;
import com.supervision.police.service.RecordSplitTypeService;
import com.supervision.springaidemo.dto.QARecordNodeDTO;
import com.supervision.springaidemo.util.RecordRegexUtil;
import com.supervision.springaidemo.util.WordReadUtil;
import com.supervision.thread.RecordSplitTypeThread;
import com.supervision.thread.RecordSplitTypeThreadPool;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.json.JSONObject;
import org.springframework.ai.chat.ChatResponse;
import org.springframework.ai.chat.messages.UserMessage;
import org.springframework.ai.chat.prompt.Prompt;
import org.springframework.ai.ollama.OllamaChatClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StopWatch;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class NoteRecordSplitServiceImpl extends ServiceImpl<NoteRecordSplitMapper, NoteRecordSplit> implements NoteRecordSplitService {

    private final NoteRecordMapper noteRecordMapper;

    private final MinioService minioService;

    private final ModelCaseMapper modelCaseMapper;

    private final MinioFileMapper minioFileMapper;

    @Autowired
    private ModelRecordTypeService modelRecordTypeService;

    @Autowired
    private RecordSplitTypeService recordSplitTypeService;


    @Override
    @Transactional(transactionManager = "dataSourceTransactionManager", rollbackFor = Exception.class)
    public String uploadRecords(NoteRecord records, List<MultipartFile> fileList) throws IOException {
        //上传文件，获取文件ids
        List<String> fileIds = new ArrayList<>();
        for (MultipartFile file : fileList) {
            String fileId = minioService.uploadFile(file);
            fileIds.add(fileId);
        }
        if (StringUtils.isNotEmpty(records.getFileIds())) {
            records.setFileIds(records.getFileIds() + "," + StringUtils.join(fileIds, ","));
        } else {
            records.setFileIds(StringUtils.join(fileIds, ","));
        }
        //修改案件的行为人、当事人
        if (StringUtils.isNotEmpty(records.getRole()) && !"嫌疑人".equals(records.getRole())) {
            ModelCase modelCase = modelCaseMapper.selectById(records.getCaseId());
            //行为人
            if ("行为人".equals(records.getRole()) && StringUtils.isEmpty(modelCase.getLawActor())) {
                modelCase.setLawActor(records.getName());
                modelCaseMapper.updateById(modelCase);
            } else if ("受害人".equals(records.getRole()) || "证人".equals(records.getRole())) {
                if (StringUtils.isEmpty(modelCase.getLawParty())) {
                    modelCase.setLawParty(records.getName());
                } else {
                    String[] split = modelCase.getLawParty().split(",");
                    if (!Arrays.asList(split).contains(records.getName())) {
                        modelCase.setLawParty(modelCase.getLawParty() + "," + records.getName());
                    }
                }
                modelCaseMapper.updateById(modelCase);
            }
        }
        int i = 0;
        if (StringUtils.isEmpty(records.getId())) {
            i = noteRecordMapper.insert(records);
        } else {
            i = noteRecordMapper.updateById(records);
        }
        //所有对话类型
        List<ModelRecordType> allTypeList = modelRecordTypeService.lambdaQuery().list();
        if (i > 0) {
            //拆分笔录
            for (MultipartFile file : fileList) {
                String context = WordReadUtil.readWord(file);
                List<QARecordNodeDTO> qaList = RecordRegexUtil.recordRegex(context, records.getName());
                for (QARecordNodeDTO qa : qaList) {
                    try {
                        NoteRecordSplit noteRecord = new NoteRecordSplit();
                        noteRecord.setCaseId(records.getCaseId());
                        noteRecord.setNoteRecordsId(records.getId());
                        noteRecord.setNoteName(file.getName());
                        noteRecord.setPersonName(records.getName());
                        noteRecord.setQuestion(qa.getQuestion());
                        noteRecord.setAnswer(qa.getAnswer());
                        noteRecord.setCreateTime(LocalDateTime.now());
                        this.save(noteRecord);
                        // 通过异步的形式提交分类
                        recordSplitTypeService.type(allTypeList, qa, noteRecord);
                    } catch (Exception e) {
                        log.error(e.getMessage(), e);
                    }
                }
            }
            return "保存成功";
        } else {
            return "保存笔录失败";
        }
    }

    @Override
    public Map<String, Object> queryRecords(NoteRecord noteRecords, Integer page, Integer size) {
        LambdaQueryWrapper<NoteRecord> wrapper = Wrappers.lambdaQuery();
        wrapper.like(StringUtils.isNotEmpty(noteRecords.getName()), NoteRecord::getName, noteRecords.getName())
                .eq(NoteRecord::getCaseId, noteRecords.getCaseId())
                .eq(NoteRecord::getDataStatus, "1");
        List<NoteRecord> list = noteRecordMapper.selectList(wrapper);
        // LinkedHashMap 保障顺序
        Map<String, List<NoteRecord>> nameMap = list.stream().filter(item -> StringUtils.isNotBlank(item.getName()))
                .collect(Collectors.groupingBy(NoteRecord::getName, LinkedHashMap::new, Collectors.toList()));
        List<NoteRecord> res = new ArrayList<>();
        for (String name : nameMap.keySet()) {
            NoteRecord noteRecord = new NoteRecord();
            noteRecord.setName(name);
            noteRecord.setChildren(nameMap.get(name));
            res.add(noteRecord);
        }
        List<NoteRecord> pager = ListUtils.Pager(size, page, res);
        for (NoteRecord person : pager) {
            for (NoteRecord noteRecord : person.getChildren()) {
                String fileIds = noteRecord.getFileIds();
                if (StringUtils.isNotEmpty(fileIds)) {
                    noteRecord.setConfessionMaterial(fileIds.split(",").length);
                }
            }
        }
        return IPages.buildDataMap(pager, res.size());
    }

    @Override
    public void delRecords(String id) {
        NoteRecord noteRecord = noteRecordMapper.selectById(id);
        noteRecord.setDataStatus(StringUtils.getUUID());
        noteRecordMapper.updateById(noteRecord);
        String fileIds = noteRecord.getFileIds();
        if (StringUtils.isNotEmpty(fileIds)) {
            //删除文件
            for (String fileId : fileIds.split(",")) {
                MinioFile minioFile = minioFileMapper.selectById(fileId);
                minioFile.setDataStatus(StringUtils.getUUID());
                minioFileMapper.updateById(minioFile);
                minioService.delFile(fileId);
            }
        }
    }

}
